import { Row, Text } from '@shopwp/wp-ui'
import to from 'await-to-js'
import difference from 'lodash/difference'
import without from 'lodash/without'
import union from 'lodash/union'
import isEmpty from 'lodash/isEmpty'

import {
	removeWebhooks,
	connectWebhooks,
	getWordPressErrorMessage,
	isWordPressError,
} from '@shopwp/api'

function findWebhooksListMinusSelected(allWebhooks, connectedWebhooks) {
	return difference(allWebhooks, connectedWebhooks)
}

function findFullWebhookFromLabel(label) {
	var found = shopwp.misc.availableWebhooks.filter(w => w.label === label)

	if (found.length) {
		return found[0]
	}

	return found
}

function allWebhooksOnlyLabels() {
	return shopwp.misc.availableWebhooks
		.map(w => (w ? w.label : false))
		.filter(Boolean)
}

function allConnectedWebhooksOnlyLabels() {
	return shopwp.general.connectedWebhooks
		.map(w => (w ? w.label : false))
		.filter(Boolean)
}

function WebhooksTool({ state, toolDispatch, adminDispatch }) {
	const { Card, CardBody, ExternalLink, Button, Notice, FormTokenField } =
		wp.components

	const { useState } = window.wp.element
	const [isRemoving, setIsRemoving] = useState(false)
	const [hasNotice, setHasNotice] = useState(false)

	const [connectedWebhooks, setConnectedWebhooks] = useState(
		shopwp.general.connectedWebhooks.length
			? allConnectedWebhooksOnlyLabels()
			: []
	)

	const [suggestions, setSuggestions] = useState(
		findWebhooksListMinusSelected(allWebhooksOnlyLabels(), connectedWebhooks)
	)

	async function removeOneOrMoreWebhooks(webhook = {}) {
		toolDispatch({
			type: 'SET_IS_BUSY',
			payload: true,
		})

		setHasNotice(false)
		setIsRemoving(true)

		const [error, resp] = await to(
			removeWebhooks({
				topics: isEmpty(webhook) ? [] : [webhook.value],
			})
		)

		toolDispatch({
			type: 'SET_IS_BUSY',
			payload: false,
		})

		setIsRemoving(false)

		if (isWordPressError(resp)) {
			setHasNotice(getWordPressErrorMessage(resp))
			return
		}

		if (error) {
			setHasNotice(JSON.stringify(error))
		} else {
			if (isEmpty(webhook)) {
				setConnectedWebhooks([])
				setSuggestions(allWebhooksOnlyLabels())
			} else {
				setConnectedWebhooks(
					without(connectedWebhooks, webhook.label).filter(Boolean)
				)

				setSuggestions(union(suggestions, [webhook.label]).filter(Boolean))
			}

			adminDispatch({
				type: 'SET_NOTICE',
				payload: {
					type: 'success',
					message: resp.data,
				},
			})
		}
	}

	async function connectOneOrMoreWebhooks(webhook = {}) {
		toolDispatch({
			type: 'SET_IS_BUSY',
			payload: true,
		})

		setHasNotice(false)

		const [error, resp] = await to(
			connectWebhooks({
				topics: isEmpty(webhook) ? [] : [webhook.value],
			})
		)

		toolDispatch({
			type: 'SET_IS_BUSY',
			payload: false,
		})

		if (isWordPressError(resp)) {
			setHasNotice(getWordPressErrorMessage(resp))
			return
		}

		if (error) {
			setHasNotice(JSON.stringify(error))
		} else {
			if (!isEmpty(webhook)) {
				var unionResult = union(connectedWebhooks, [webhook.label]).filter(
					Boolean
				)

				var withoutResult = without(suggestions, webhook.label).filter(Boolean)

				setConnectedWebhooks(unionResult)
				setSuggestions(withoutResult)
			}

			adminDispatch({
				type: 'SET_NOTICE',
				payload: {
					type: 'success',
					message: resp.data,
				},
			})
		}
	}

	function onRemove() {
		removeOneOrMoreWebhooks()
	}

	function onConnect() {
		connectOneOrMoreWebhooks()
	}

	function onChange(latestSelection) {
		if (!latestSelection.length) {
			// Remove all
			removeOneOrMoreWebhooks()
			return
		}

		/*
		
		Is the latest selection already included in the list of connected webhooks?

		If so, we know the user has chosen to removed it.

		Otherwise, we know they're adding it.
		
		*/

		// Create a new array and exclude any values in connectedWebhooks from latestSelection. This should return to us any "added" products
		var added = difference(latestSelection, connectedWebhooks)
		var removed = difference(connectedWebhooks, latestSelection)

		if (removed.length) {
			var found = findFullWebhookFromLabel(removed[0])

			removeOneOrMoreWebhooks(found)
		} else if (added.length) {
			var found = findFullWebhookFromLabel(added[0])

			connectOneOrMoreWebhooks(found)
		}
	}

	return (
		<Card>
			<CardBody>
				<Row align='space-between'>
					<Text variant='subtitle.small'>
						Webhooks
						<ExternalLink href='https://wpshop.io' className='swp-webhook-link'>
							{shopwp.t.l.learnMore}
						</ExternalLink>
					</Text>
					{connectedWebhooks.length ? (
						<Button
							isPrimary
							variant='secondary'
							isDestructive={true}
							text='Disconnect all from this site'
							onClick={onRemove}
							disabled={state.isBusy}
							isBusy={isRemoving}
							className='swp-remove-webhooks'
						/>
					) : null}
				</Row>
				<FormTokenField
					disabled={state.isBusy}
					value={connectedWebhooks}
					label='Connected webhooks'
					suggestions={suggestions}
					placeholder='Choose a webhook to connect'
					onChange={onChange}
					__experimentalExpandOnFocus
					__experimentalShowHowTo={false}
				/>
				{hasNotice ? (
					<Row>
						<Notice status='error' isDismissible={false}>
							{hasNotice}
						</Notice>
					</Row>
				) : null}
			</CardBody>
		</Card>
	)
}

export default WebhooksTool
